//::///////////////////////////////////////////////
//:: Lightning Bolt
//:: NW_S0_LightnBolt
//:: Copyright (c) 2001 Bioware Corp.
//:://////////////////////////////////////////////
/*
    Does 1d6 per level in a 5ft tube for 30m
*/
//:://////////////////////////////////////////////
//:: Created By: Noel Borstad
//:: Created On:  March 8, 2001
//:://////////////////////////////////////////////
//:: Last Updated By: Preston Watamaniuk, On: May 2, 2001

#include "X0_I0_SPELLS"
#include "ginc_debug"
#include "x0_i0_position"

#include "jx_inc_magic"
#include "x2_inc_spellhook"

void mainSpell();
void main()
{
	AssignCommand(JXGetCaster(), mainSpell());
}

void mainSpell()
{
	if (!X2PreSpellCastCode()) { JXPostSpellCastCode(); return; }


    //Declare major variables
    int nAge = GetHitDice(OBJECT_SELF);
    int nDamage, nDC, nDamStrike;
    float fDelay;
    object oTarget;
    effect eVis, eBreath;
    //Use the HD of the creature to determine damage and save DC
    if (nAge <= 6) //Wyrmling
    {
        nDamage = d4(2);
        nDC = 13;
    }
    else if (nAge >= 7 && nAge <= 9) //Very Young
    {
        nDamage = d4(4);
        nDC = 14;
    }
    else if (nAge >= 10 && nAge <= 12) //Young
    {
        nDamage = d4(6);
        nDC = 17;
    }
    else if (nAge >= 13 && nAge <= 15) //Juvenile
    {
        nDamage = d4(8);
        nDC = 18;
    }
    else if (nAge >= 16 && nAge <= 18) //Young Adult
    {
        nDamage = d4(10);
        nDC = 22;
    }
    else if (nAge >= 19 && nAge <= 21) //Adult
    {
        nDamage = d4(12);
        nDC = 23;
    }
    else if (nAge >= 22 && nAge <= 24) //Mature Adult
    {
        nDamage = d4(12);
        nDC = 26;
    }
    else if (nAge >= 25 && nAge <= 27) //Old
    {
        nDamage = d4(16);
        nDC = 27;
    }
    else if (nAge >= 28 && nAge <= 30) //Very Old
    {
        nDamage = d4(18);
        nDC = 30;
    }
    else if (nAge >= 31 && nAge <= 33) //Ancient
    {
        nDamage = d4(20);
        nDC = 31;
    }
    else if (nAge >= 34 && nAge <= 37) //Wyrm
    {
        nDamage = d4(24);
        nDC = 34;
    }
    else if (nAge > 37) //Great Wyrm
    {
        nDamage = d4(24);
        nDC = 36;
    }
	
	oTarget = JXGetSpellTargetObject();	
	effect eAcidBeam = EffectVisualEffect(VFX_BEAM_GREEN_DRAGON_ACID);
    PlayDragonBattleCry();  //What does this do?  Mysteries abound in the land of Faerun.
	PlayCustomAnimation(OBJECT_SELF, "Una_breathattack01", 0, 0.7f);
    //Get first target in spell area
	location lLocation = GetLocation(oTarget);
	
	location lNewLocation = GetAheadLocation(OBJECT_SELF, 60.0f);
	object oEndTarget = CreateObject(OBJECT_TYPE_PLACEABLE, "plc_ipoint ", lNewLocation);
	
	DelayCommand(1.0f, DestroyObject(oEndTarget));
	
	DelayCommand(0.5f, JXApplyEffectToObject(DURATION_TYPE_TEMPORARY, eAcidBeam, oEndTarget, 4.0f));
	
    oTarget = GetFirstObjectInShape(SHAPE_SPELLCONE, 60.0, lLocation, TRUE);
    while(GetIsObjectValid(oTarget))
    {
        if(oTarget != OBJECT_SELF && spellsIsTarget(oTarget, SPELL_TARGET_STANDARDHOSTILE, OBJECT_SELF))
        {
            //Reset the damage to full
            nDamStrike = nDamage;
            //Fire cast spell at event for the specified target
            SignalEvent(oTarget, EventSpellCastAt(OBJECT_SELF, SPELLABILITY_DRAGON_BREATH_ACID));
            //Adjust the damage based on the Reflex Save, Evasion and Improved Evasion.
            //Determine effect delay
            fDelay = GetDistanceBetween(OBJECT_SELF, oTarget)/20.0f;
            if(MySavingThrow(SAVING_THROW_REFLEX, oTarget, nDC, SAVING_THROW_TYPE_ACID))
            {
                nDamStrike = nDamStrike/2;
                if(GetHasFeat(FEAT_EVASION, oTarget) || GetHasFeat(FEAT_IMPROVED_EVASION, oTarget))
                {
                    nDamStrike = 0;
                }
            }
            else if(GetHasFeat(FEAT_IMPROVED_EVASION, oTarget))
            {
                nDamStrike = nDamStrike/2;
            }
            if (nDamStrike > 0)
            {
                //Set Damage and VFX
                eBreath = EffectDamage(nDamStrike, DAMAGE_TYPE_ACID);
                eVis = EffectVisualEffect(VFX_IMP_ACID_L);
                //Apply the VFX impact and effects
                DelayCommand(1.0, JXApplyEffectToObject(DURATION_TYPE_TEMPORARY, eVis, oTarget, 3.0 ));
                DelayCommand(1.5, JXApplyEffectToObject(DURATION_TYPE_TEMPORARY, eVis, oTarget, 3.0 ));
                DelayCommand(2.0, JXApplyEffectToObject(DURATION_TYPE_TEMPORARY, eVis, oTarget, 3.0 ));
                DelayCommand(1.0, JXApplyEffectToObject(DURATION_TYPE_INSTANT, eBreath, oTarget));
             }
        }
        //Get next target in spell area
        oTarget = GetNextObjectInShape(SHAPE_SPELLCONE, 60.0, lLocation, TRUE);
    }

	JXPostSpellCastCode();
}
