//::///////////////////////////////////////////////
//:: Wild Mage Spell - Vortex
//::///////////////////////////////////////////////
/*
Description modified from AD&D 2nd Edition - "Tome of Magic"

Vortex

Caster Level(s): Wizard / Sorcerer 5
Innate Level: 5
School: Evocation
Descriptor(s): Wild Magic
Component(s): Verbal, Somatic
Range: Medium
Area of Effect / Target: 8' diameter circle / caster’s familiar
Duration: 1d4 + 1 round / 2 lvl
Save: Fortitude 1/2
Spell Resistance: Yes

     This spell takes advantage of the magical link between 
a wizard and his familiar.  The wild mage manipulates this 
connection transforming his familiar into a mass of swirling 
raw magical energy.  Once the transformation is complete all 
semblance of personality and loyalty in this once faithful 
companion is lost.  Each round the caster can attempt to 
possess the Vortex temporarily controlling its movement.  
However, his control is not perfect and there is a 50% 
chance of losing control each round.  If loses control, or 
does not take control in the first place, the Vortex will 
move around randomly.   The link between the caster and the 
Vortex is not as strong as it was with his familiar and if 
the Vortex moves beyond 30 meters he will be unable to 
possess it.  At the end of the spell the casters familiar 
reforms but may suffer damage from the transition.

	The vortex is composed of raw magical energy. Nonmagical
creatures within 7.5 feet of the vortex suffer ld4 points of 
damage per level of the caster. Magical creatures and 
spellcasters suffer ld6 points of damage per level of the 
caster. Creatures struck are allowed a Fortitude saving 
throw vs. spells to suffer only half damage. Each time a 
creature is struck, there is a 5% chance that the vortex 
will explode in a wild surge. When this happens a random 
spell between levels 1 and 6 is cast targeting a random 
creature within 7.5 feet of the vortex. This spell is 
guaranteed to cause a wild surge.  The probability of each 
level being cast is as follows.
Level 1 = 10%
Level 2 = 20%
Level 3 = 25%
Level 4 = 20%
Level 5 = 15%
Level 6 = 10%
If the vortex causes a wild surge, the spell ends 
immediately.




2d2f_sp_Vortex.nss
*/
//::///////////////////////////////////////////////
//:: Created By: 2DruNk2Frag
//:: Created On: 03-16-08
//:: Last Update: 03-22-08
//::///////////////////////////////////////////////

#include "2d2f_includes"
#include "jx_inc_magic"
#include "x2_inc_spellhook" 
#include "jx_inc_magic_info"
#include "nwn2_inc_spells"
#include "nw_i0_spells"
#include "gui_bhvr_inc"



void mainSpell();
void main()
{
	AssignCommand(JXGetCaster(), mainSpell());
}

void mainSpell()
{
	object oCaster = JXGetCaster();
	int nWMLvl = GetLevelByClass(CLASS_WILDMAGE, oCaster);
	if((nWMLvl >= 7) || (GetObjectType(oCaster) == OBJECT_TYPE_PLACEABLE))
	{
/* 
  Spellcast Hook Code 
  Added 2003-06-23 by GeorgZ
  If you want to make changes to all spells,
  check x2_inc_spellhook.nss to find out more
  
*/

    if (!X2PreSpellCastCode())
    {
	// If code within the PreSpellCastHook (i.e. UMD) reports FALSE, do not run this spell
        { JXPostSpellCastCode(); return; }
    }
	//:://////////////////////////////////////////
	
	object oTarget = JXGetSpellTargetObject();
	int nLvl = JXGetCasterLevel();
	int nDur = d4(1) + (nLvl / 2);
	float fDuration = RoundsToSeconds(nDur);


    fDuration = ApplyMetamagicDurationMods(fDuration);
    int nDurType = ApplyMetamagicDurationTypeMods(DURATION_TYPE_TEMPORARY);
	
	effect eAOE = EffectAreaOfEffect(AOE_PER_VORTEX,"2d2f_sp_vortex_enter","2d2f_sp_vortex_heart","");

	effect eVis = EffectVisualEffect(VFX_DUR_VORTEX);
	effect eLink = EffectLinkEffects(eVis,eLink);
	eLink = EffectLinkEffects(eAOE,eLink);
	eLink = SetEffectSpellId(eLink,SPELL_VORTEX);
		
	effect eBeam = EffectBeam(VFX_BEAM_VORTEX,oCaster,BODY_NODE_HAND,FALSE);

	if(GetName(oTarget) == GetAnimalCompanionName(oCaster))
	{
		SetLocalObject(oCaster,"oFamiliar",oTarget);
		DelayCommand(fDuration,DeleteLocalObject(oCaster,"oFamiliar"));
		SetImmortal(oTarget, TRUE);
		DelayCommand(fDuration,SetImmortal(oTarget, FALSE));
		DelayCommand(fDuration,ClearAllActions());
		
		SetAssociateState(NW_ASC_MODE_STAND_GROUND, TRUE, oTarget);
		DelayCommand(fDuration,SetAssociateState(NW_ASC_MODE_STAND_GROUND, FALSE, oTarget));
	
		SetAssociateState(NW_ASC_MODE_PUPPET, TRUE, oTarget);
		DelayCommand(fDuration,SetAssociateState(NW_ASC_MODE_PUPPET, FALSE, oTarget));
	
		
		AssignCommand(oTarget,ActionRandomWalk());
	
		ApplyEffectToObject(DURATION_TYPE_TEMPORARY,eBeam,oTarget,3.0);
		ApplyEffectToObject(nDurType,eLink,oTarget,fDuration);

	}
	else
	{
		SendMessageToPC(oCaster,"The spell vortex only works on your familiar.");
	}
		
	}
	else
	{
	SendMessageToPC(oCaster, "You must be at least a level 7 Wild Mage to use this spell");
	}
	
	
	JXPostSpellCastCode();
	return;
}